@@ -39,7 +39,12 @@ from networking_bgpvpn.neutron.services.common import constants
 
 LOG = log.getLogger(__name__)
 
+# ("BGPVPN" is the string to match as the first part of the
+# service_provider configuration: "BGPVPN:Dummy:networking_bgpvpn ...")
+SERVICE_PROVIDER_TYPE = "BGPVPN"
 
+
+@registry.has_registry_receivers
 class BGPVPNPlugin(bgpvpn.BGPVPNPluginBase,
                    bgpvpn_rc.BGPVPNRoutesControlPluginBase):
 
@@ -50,12 +55,12 @@ class BGPVPNPlugin(bgpvpn.BGPVPNPluginBase,
         # service_provider definitions:
         service_type_manager = st_db.ServiceTypeManager.get_instance()
         service_type_manager.add_provider_configuration(
-            bgpvpn_def.LABEL,
+            SERVICE_PROVIDER_TYPE,
             pconf.ProviderConfiguration('networking_bgpvpn'))
 
         # Load the default driver
-        drivers, default_provider = service_base.load_drivers(bgpvpn_def.LABEL,
-                                                              self)
+        drivers, default_provider = service_base.load_drivers(
+            SERVICE_PROVIDER_TYPE, self)
         LOG.info("BGP VPN Service Plugin using Service Driver: %s",
                  default_provider)
         self.driver = drivers[default_provider]
@@ -64,9 +69,6 @@ class BGPVPNPlugin(bgpvpn.BGPVPNPluginBase,
             LOG.warning("Multiple drivers configured for BGPVPN, although"
                         "running multiple drivers in parallel is not yet"
                         "supported")
-        registry.subscribe(self._notify_adding_interface_to_router,
-                           resources.ROUTER_INTERFACE,
-                           events.BEFORE_CREATE)
 
     @property
     def supported_extension_aliases(self):
@@ -74,11 +76,12 @@ class BGPVPNPlugin(bgpvpn.BGPVPNPluginBase,
         exts += self.driver.more_supported_extension_aliases
         return exts
 
+    @registry.receives(resources.ROUTER_INTERFACE, [events.BEFORE_CREATE])
     def _notify_adding_interface_to_router(self, resource, event, trigger,
-                                           **kwargs):
-        context = kwargs.get('context')
-        network_id = kwargs.get('network_id')
-        router_id = kwargs.get('router_id')
+                                           payload):
+        context = payload.context
+        network_id = payload.metadata.get('network_id')
+        router_id = payload.resource_id
         try:
             routers_bgpvpns = self.driver.get_bgpvpns(
                 context,
@@ -158,7 +161,7 @@ class BGPVPNPlugin(bgpvpn.BGPVPNPluginBase,
                     raise n_exc.BadRequest(resource='bgpvpn', msg=msg)
 
     def get_plugin_type(self):
-        return bgpvpn_def.LABEL
+        return bgpvpn_def.ALIAS
 
     def get_plugin_description(self):
         return "Neutron BGPVPN Service Plugin"
@@ -261,6 +264,13 @@ class BGPVPNPlugin(bgpvpn.BGPVPNPluginBase,
                 raise bgpvpn_rc.BGPVPNPortAssocRouteNoSuchBGPVPN(
                     bgpvpn_id=route['bgpvpn_id'])
 
+            assoc_bgpvpn = self.get_bgpvpn(context, bgpvpn_id)
+            if route_bgpvpn['type'] != assoc_bgpvpn['type']:
+                raise bgpvpn_rc.BGPVPNPortAssocRouteBGPVPNTypeDiffer(
+                    route_bgpvpn_type=route_bgpvpn['type'],
+                    bgpvpn_type=assoc_bgpvpn['type']
+                    )
+
             assoc_tenant_id = port_association.get('project_id')
             if assoc_tenant_id is None:
                 # update, rather than create, we need to retrieve the tenant
